// cabfile.cpp
//
// Copyright (C) 2000  Robert Barron, KA5WSS
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "cabfile.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>         // For isdigit()
#include <time.h>
#include <string.h>

#define MAX_ARRL_LINE_SIZE    100
#define MAX_ARRL_CALL_SIZE    15

char *RCabFile::modes[] = {"?MODE?", "CW", "PH", "RY", "PH", "FM", NULL};
char *RCabFile::bands[] = {NULL, "1800", "3500", "7000", "10000", "14000", "18068",
                         "21000", "24890", "28000", "A", "76", "B", "C", "D", "9",
                         "E", "F", "G", "H", "I", "J", "K", "M", "N", "P", "R", NULL,
                         NULL, NULL, "L", NULL};
char *RCabFile::contests[] = {NULL, "ARRL-SS-", "ARRL-DX-", "CQ-WW-", "CQ-WPX-", "IARU-HF",
                         "ARRL-10", "ARRL-160", "CQ-160-", NULL, "CA-QSO-PARTY", "ARRL-DX-",
                         NULL, "ARRL-VHF-", NULL, NULL, "NAQP-", NULL, NULL, NULL, NULL,
                         "NA-SPRINT-", NULL, NULL, NULL, NULL, NULL, NULL, NULL, "STEW-PERRY",
						 NULL, NULL, NULL};
#define MAX_CAB_CONTEST		STEW_PERRY_CONTEST

char *RCabFile::categories[] = {"SINGLE-OP", "SINGLE_OP_ASSISTED", "MULTI-ONE",
                        "MULTI-TWO", "MULTI-MULTI", "SINGLE-OP-QRP-PORTABLE", "SINGLE-OP",
                        "SINGLE-OP", "ROVER", "MULTI-LIMTED", NULL};
#define MAX_CAB_CATEGOTY CAT_LIMITED_MULTI

char *RCabFile::powerCategories[] = {"?PWR?", "QRP", "LOW", "HIGH", NULL};

RCabFile::RCabFile(char *filename)
{
    RCabFile::open(filename);
}

boolean RCabFile::open(char *filename, int mode)
{
    if (!RFile::open(filename, mode))
        return FALSE;

    // There is no header to read.  Must make sure to specify the
    //   contest type through a command line option!!!!

    // Set some options to defaults so that required fields are
    // not NULL.
    strcpy(header.callsign, "NOCALL");
    
    location = 0;

    return TRUE;
}

boolean RCabFile::close()
{
    char buffer[MAX_ARRL_LINE_SIZE];

    // If this file was opened output, then we need to write the end
    // of file field.
    strcpy(buffer, CAB_FIELD_END);
	strcat(buffer, "\r\n");
    RFile::write(buffer,strlen(buffer));

    // Now make sure we close the file too.
    return RFile::close();
}

boolean RCabFile::nextQso(qso_data *qso)
{
    return FALSE;
/*    unsigned char count;
    char buffer[MAX_ARRL_LINE_SIZE];
    char *tmpPtr;

    qso->init();

    RFile::getString((char *)buffer, MAX_ARRL_LINE_SIZE);
    if (RFile::eof())
        return FALSE;
    
    // Store band into qso_data
    tmpPtr = strtok(buffer, " ");
    for (count = 1; count <= MAX_BAND; count++)
        if ((bands[count] != NULL) && (strcmp(bands[count], tmpPtr) == 0))
            qso->setBand(count);
            
    // Store mode into qso_data
    tmpPtr = strtok(NULL, " ");
    for (count = 1; modes[count] != NULL; count++)
        if (strcmp(modes[count], tmpPtr) == 0)
            qso->setMode(count);
    // Special case.  TR Log outputs "PH" instead of
    //  SSB as the mode for phone.
    if ((qso->getMode() == 0) && (strcmp(tmpPtr, "PH") == 0))
        qso->setMode(SSB_MODE);

    // Convert the verbose time to time_t format.
    {
        int month, day, year, hours, minutes;

        day = atoi(strtok(NULL, "/"));
        month = atoi(strtok(NULL, "/")) - 1;
        year = atoi(strtok(NULL, " "));
        tmpPtr = strtok(NULL, " ");
        minutes = atoi(&tmpPtr[2]);
        tmpPtr[2] = (char)NULL;
        hours = atoi(tmpPtr);
        qso->setTime(year, month, day, hours, minutes);
    }
        
    // Get QSO number.  We don't need it though.
    tmpPtr = strtok(NULL, " ");

    if (header.contest == ARRL_SS_CONTEST)
    {
        tmpPtr = strtok(NULL, " ");       // Get precedence sent for SS
        tmpPtr = strtok(NULL, " ");       // Get callsign sent
    }
    else
        // Copy callsign of station worked to log_data struct.
        qso->setCallsign(strtok(NULL, " "));

    switch (header.contest)
    {
        case ARRL_SS_CONTEST:
            tmpPtr = strtok(NULL, " ");                // Get check sent
            tmpPtr = strtok(NULL, " ");                // Get section sent
            qso->setSerialNumber(strtok(NULL, " "));
            tmpPtr = strtok(NULL, " ");
            qso->setSSPrecedence(*tmpPtr);             // Get precedence received
            qso->setCallsign(strtok(NULL, " "));
            qso->setSSCheck(strtok(NULL, " "));
            qso->setSection(strtok(NULL, " "));
            break;
        case ARRL_DX_CONTEST_DX:
        case ARRL_10M_CONTEST:
            tmpPtr = strtok(NULL, " ");                 // Get RST sent
            tmpPtr = strtok(NULL, " ");                 // Get info sent
            qso->setReceivedRST(strtok(NULL, " "));
            qso->setInfo(strtok(NULL, " "));
            break;
        case IARU_CONTEST:
            tmpPtr = strtok(NULL, " ");                 // Get RST sent
            tmpPtr = strtok(NULL, " ");                 // Get info sent
            qso->setReceivedRST(strtok(NULL, " "));     // Get RST received
            tmpPtr = strtok(NULL, " ");
            if (isdigit(tmpPtr[0]))
                qso->setItuZone(tmpPtr);
            else
                qso->setInfo(tmpPtr);                   // Save IARU HQ Name
            break;
        case ARRL_DX_CONTEST:
        case ARRL_160M_CONTEST:
            tmpPtr = strtok(NULL, " ");                 // Get RST sent
            tmpPtr = strtok(NULL, " ");                 // Get next field. May be info or RST.
            if (isdigit(*tmpPtr))
                qso->setReceivedRST(tmpPtr);            // Not the expected info, so must be the
                                                        // received RST.  Record it.
            qso->setInfo(strtok(NULL, " "));
            break;
        case FD_CONTEST:
            tmpPtr = strtok(NULL, " ");                 // Skip over sent tx count and cat.
            tmpPtr = strtok(NULL, " ");                 // Skip over sent section
            tmpPtr = strtok(NULL, " ");                 // Get tx count and category
            qso->setFdCategory(tmpPtr[strlen(tmpPtr) - 1]);
            tmpPtr[strlen(tmpPtr) - 1] = (char)NULL;
            qso->setFdTransmitters(tmpPtr);
            qso->setSection(strtok(NULL, " "));       // Get section.
            break;
        case ARRL_VHF_QSO_PARTY:
            buffer[52] = (char)NULL;
            qso->setInfo(&buffer[48]);
            break;
        case NA_QSO_PARTY_CONTEST:
            qso->setName(strtok(NULL, " "));            // Get name received.
            qso->setInfo(strtok(NULL, " "));            // Get state received.
            break;
        case NA_SPRINT_CONTEST:
            qso->setSerialNumber(strtok(NULL, " "));
            qso->setName(strtok(NULL, " "));            // Get name received.
            qso->setInfo(strtok(NULL, " "));            // Get state received.
            break;
        default:
            break;
    };

    tmpPtr = strtok(NULL, " ");
    if ((strchr(tmpPtr, '\r') == NULL)
        && ((strlen(tmpPtr) != 1) || (!isdigit(*tmpPtr))))  // Is this a multiplier?
    {
        qso->setMultCount(1);
        qso->setQsoPoints(strtok(NULL, " "));
    }
    else
        qso->setQsoPoints(tmpPtr);
    
    location++;
    return TRUE;*/
}

boolean RCabFile::writeHeader(header_data newheader)
{
    char buffer[MAX_CAB_LINE_SIZE];
    char tempStr[20];

    /* Test to see if the contest is supported by this ARRL write code. */
    /*  Return false if it is not supported. */
    if ((newheader.contest == 0)
         || (newheader.contest > MAX_CAB_CONTEST)
         || (contests[newheader.contest] == NULL))
        return FALSE;

    sprintf(buffer, "%s %s\r\n", CAB_FIELD_START, CAB_FILE_VER);
    RFile::write(buffer,strlen(buffer));

    // For SS contest logs we need the section field.  If the log does not have this
    // info print out a holder for it anyway.
    if (newheader.contest == ARRL_DX_CONTEST_DX)
        sprintf(buffer, "%s %s\r\n", CAB_FIELD_SECTION, "DX");
    else if (strlen(newheader.ss.section) != 0)
        sprintf(buffer, "%s %s\r\n", CAB_FIELD_SECTION, newheader.ss.section);
    else
        sprintf(buffer, "%s %s\r\n", CAB_FIELD_SECTION, "");
    RFile::write(buffer,strlen(buffer));

    strcpy(tempStr, contests[newheader.contest]);
    // If the contest name ends in '-' then we need to add some more information, such
    // as the contest mode.
    if (tempStr[strlen(tempStr) - 1] == '-')
	{
		if (newheader.contest == ARRL_VHF_QSO_PARTY)
			// TBD: Need to come up with a simple way to include the month
			strcat(tempStr, "?MON?");
		else
			// These contests include the mode as part of the contest title.
	        strcat(tempStr, modes[newheader.mode]);
	}
    sprintf(buffer, "%s %s\r\n", CAB_FIELD_CONTEST, tempStr);
    RFile::write(buffer,strlen(buffer));

    sprintf(buffer, "%s %s\r\n", CAB_FIELD_CALLSIGN, newheader.callsign);
    RFile::write(buffer,strlen(buffer));

    char szBandCat[5];
    if (newheader.contest == ARRL_10M_CONTEST)
        strcpy(szBandCat, "10M");
    else if (newheader.contest == ARRL_160M_CONTEST)
        strcpy(szBandCat, "160M");
    else
        strcpy(szBandCat, "ALL");

    sprintf(buffer, "%s %s %s %s\r\n", CAB_FIELD_CATEGORY, categories[newheader.category],
            szBandCat, powerCategories[newheader.powercat]);
    RFile::write(buffer,strlen(buffer));

    sprintf(buffer, "%s %s\r\n", CAB_FIELD_SCORE, "");
    RFile::write(buffer,strlen(buffer));

    sprintf(buffer, "%s %s\r\n", CAB_FIELD_OPERATORS, "");
    RFile::write(buffer,strlen(buffer));

    sprintf(buffer, "%s %s\r\n", CAB_FIELD_CLUB, newheader.club);
    RFile::write(buffer,strlen(buffer));

    sprintf(buffer, "%s %s\r\n", CAB_FIELD_NAME, newheader.name);
    RFile::write(buffer,strlen(buffer));

    sprintf(buffer, "%s %s\r\n", CAB_FIELD_ADDRESS, newheader.address);
    RFile::write(buffer,strlen(buffer));
    sprintf(buffer, "%s %s, %s %s\r\n", CAB_FIELD_ADDRESS, newheader.city,
                                        newheader.state, newheader.zip);
    RFile::write(buffer,strlen(buffer));

    sprintf(buffer, "%s\r\n", CAB_FIELD_SOAPBOX);
    RFile::write(buffer,strlen(buffer));

    RQsoFile::storeHeader(newheader);
    return TRUE;
}

boolean RCabFile::writeQso(qso_data *qso)
{
    char buffer[MAX_CAB_LINE_SIZE];
    char buffer2[MAX_CAB_LINE_SIZE];
    struct tm *dosTime;
    char multChar = '*';
    time_t tempTime;
    char szFreq[6];
    char szMyZone[5];
    char szTheirZone[5];
	char szTheirSerialNum[6];
	char szSentRST[4];
	char szRcvdRST[4];

    location++;

	if (qso->getBand() < BAND_50MHZ)
	{
		// If the QSO does not include an exact frequency use the band field to create one.
	    if (qso->getFreq() != 0)
		    sprintf(szFreq, "%d", qso->getFreq() / 10);
	    else
			strcpy(szFreq, bands[qso->getBand()]);
	}
	else
		// The Cabrillo format always uses band indicators for VHF, never freq.
		strcpy(szFreq, bands[qso->getBand()]);

	// Break out the time into elements that can be printed.
    tempTime = qso->getTime();
    dosTime = gmtime(&tempTime);

    sprintf(buffer, CAB_QSO_TIME, szFreq, modes[qso->getMode()],
                dosTime->tm_year + 1900, dosTime->tm_mon + 1, dosTime->tm_mday,
                dosTime->tm_hour, dosTime->tm_min);

    if (qso->getMultCount() == 0)
        multChar = ' ';

    switch(header.contest)
    {
        case ARRL_SS_CONTEST:
		{
			char szCheck[3];
			char cPrecedence;
			char szSection[5];

			if (header.ss.precedence == 0)
				cPrecedence = '?';
			else
				cPrecedence = header.ss.precedence;

			if (header.ss.check == 0)
				strcpy(szCheck, "??");
			else
				sprintf(szCheck, "%02d", header.ss.check);

			if (strlen(header.ss.section) == 0)
				strcpy(szSection, "???");
			else
				strcpy(szSection, header.ss.section);

            sprintf(buffer2, CAB_QSO_SS, header.callsign, location, cPrecedence,
                             szCheck, szSection, qso->getCallsign(),
                             qso->getSerialNumber(), qso->getSSPrecedence(),
                             qso->getSSCheck(), qso->getSection());
            break;
		}
        case ARRL_DX_CONTEST:
        case ARRL_10M_CONTEST:
        case ARRL_160M_CONTEST:
        case CQ_160M_CONTEST:
        case STEW_PERRY_CONTEST:
            sprintf(buffer2, CAB_QSO_OTHER, header.callsign,
                qso->getSentRST(), "???", qso->getCallsign(), qso->getReceivedRST(),
                qso->getInfo());
            break;
        case CQ_WPX_CONTEST:
            sprintf(szTheirSerialNum, "%5d", qso->getSerialNumber());
            sprintf(buffer2, CAB_QSO_OTHER, header.callsign,
                qso->getSentRST(), "???", qso->getCallsign(), qso->getReceivedRST(),
                szTheirSerialNum);
            break;
        case CQ_WW_CONTEST:
            sprintf(szTheirZone, "%02d", qso->getCqZone());
            sprintf(buffer2, CAB_QSO_OTHER, header.callsign,
                qso->getSentRST(), "???", qso->getCallsign(), qso->getReceivedRST(),
                szTheirZone);
            break;
        case IARU_CONTEST:
			if (header.zone != 0)
				sprintf(szMyZone, "%02d", header.zone);
			else
				strcpy(szMyZone, "??");
            if (qso->getItuZone() > 0)
            {
                sprintf(szTheirZone, "%02d", qso->getItuZone());
                sprintf(buffer2, CAB_QSO_OTHER, header.callsign, qso->getSentRST(),
                         szMyZone, qso->getCallsign(), qso->getReceivedRST(),
                         szTheirZone);
            }
            else
                sprintf(buffer2, CAB_QSO_OTHER, header.callsign, qso->getSentRST(),
                         szMyZone, qso->getCallsign(), qso->getReceivedRST(),
                         qso->getInfo());
            break;
        case ARRL_DX_CONTEST_DX:
            sprintf(buffer2, CAB_QSO_OTHER, header.callsign, qso->getSentRST(),
				"????", qso->getCallsign(), qso->getReceivedRST(), qso->getInfo());
            break;
        case FD_CONTEST:
            {
                int transCount = header.fd.transmitters;

                if (transCount == 0)
                    transCount = 1;
                    
                sprintf(buffer2, "%-14s %2d%c %-4s %2d%c %-4s %1d", qso->getCallsign(),
                        transCount, header.fd.category + 'A', header.fd.section,
                        qso->getFdTransmitters(), qso->getFdCategory(),
                        qso->getSection(), qso->getQsoPoints());
            }
            break;
        case ARRL_VHF_QSO_PARTY:

			// VHF contest QSO's may not include RST exchanges.  We use strings here for
			// RST so that we get blank spaces for no exchange instead of a "0", which
			// is crpytic at best.
			if (qso->getReceivedRST() == 0)
				strcpy(szRcvdRST, "   ");
			else
				sprintf(szRcvdRST, "%-3d", qso->getReceivedRST());
			if (qso->getSentRST() == 0)
				strcpy(szSentRST, "   ");
			else
				sprintf(szSentRST, "%-3d", qso->getSentRST());

            sprintf(buffer2, CAB_QSO_VHF, header.callsign, szSentRST, header.grid,
					qso->getCallsign(), szRcvdRST, qso->getInfo());
            break;
        case NA_QSO_PARTY_CONTEST:
            sprintf(buffer2, CAB_QSO_QP, header.callsign, "", "????", "????",
					qso->getCallsign(), "", qso->getName(),
					qso->getInfo());
            break;
        case CAL_QP_CONTEST:
        case NA_SPRINT_CONTEST:
            sprintf(buffer2, CAB_QSO_SPRINT, header.callsign, location, "????", "????",
					qso->getCallsign(), qso->getSerialNumber(), qso->getName(),
					qso->getInfo());
            break;
        default:
            return FALSE;
    }
    strcat(buffer, buffer2);

    strcat(buffer, "\r\n");
    RFile::write(buffer,strlen(buffer));

    return TRUE;
}

