// logcomp.cpp
//
// Copyright (C) 2000  Robert Barron, KA5WSS
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#if defined(WIN32)
#include <windows.h>
#endif
#include <stdio.h>
#include <stdlib.h>

#include "qsofile.h"
#include "arrlfile.h"
#include "trfile.h"

#define STRING_BUFFER_SIZE    2048
#define MAX_NUM_OF_LOGS       3

RQsoFile *logs[MAX_NUM_OF_LOGS];
RFile *outFile;

static char *contestNames[] = {NULL, "Sweepstakes", "ARRL DX Contest",
                                "CQ WW Contest", "CQ WPX Contest", "IARU HF Championship",
                                "ARRL 10M Contest", "ARRL 160M Contest",
                                "CQ 160M Contest", "Field Day",
                                "California QSO Party", "ARRL DX Contest (DX)", "WAE Contest",
                                "VHF QSO Party", "General QSO", "All Asian",
                                "NA QSO Party", "GridLoc",
                                "YO DX Contest", "KCJ Contest",
                                "Ten Ten QSO Party", "NA Sprint",
                                "Scandinavian Contest", "TOEC WW GRID CONTEST", NULL};
#define MAX_LOGCOMP_CONTEST  TOEC_CONTEST

static char *bandName[] = {NULL, "160M", "80M", "40M", "30M", "20M", "17M", "15M", "12M",
                          "10M", "6M", "??M", "2M", "1.25M", "70cm", "35cm", "23cm", "13cm",
                          "9cm", "6cm", "3cm", "2cm", "1.25cm", "6mm", "2.5mm", "2mm", "1mm",
                          "SAT", "PACK", "NOV", "LIGHT", NULL};

#if !defined (LC_CMD_LINE)    
	char *exeName = "logcomp";
#elif defined(LC_NT)
	char *exeName = "clogcomp";
#endif

#if defined(LC_CMD_LINE)
	static void ShowOptions();
#endif

struct optType {
    int            timeSlice;              // Time slice in minutes
    unsigned char reportType;             // USES REPORT_ #define values
} options;

#define REPORT_DEFAULT       0
#define REPORT_QSOS          1
#define REPORT_BANDS         2

#if !defined(LC_CMD_LINE)
static int  GetInputFileName(char *);
static int  GetOutputFileName(char *, int *);
static void FilePrefix(char *, char*);
#endif
static int  LogType(char *);
static void ShutDown();
static void ProcessOptions(char *, struct optType *);

#if defined(LC_GUI) && defined(LC_NT)
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPTSTR lpCmdLine, int nCmdShow)
#else
int main(int argc, char *argv[])
#endif                   
{
#if !defined(LC_CMD_LINE)    
    boolean bAnotherLog = IDYES;
    int  OutputFileType;
#else
#if !defined(LC_GUI) || !defined(LC_NT)
    char lpCmdLine[STRING_BUFFER_SIZE];
#endif
#endif
    time_t currTime;
    time_t nextTime;
    
    char InputFileName1[STRING_BUFFER_SIZE];
    char InputFileName2[STRING_BUFFER_SIZE];
    char OutputFileName[STRING_BUFFER_SIZE];
    char displayBuffer[80];
#if !defined (LC_CMD_LINE)    
    char *progName = "LogComp";
#endif
    
    qso_data     qso[MAX_NUM_OF_LOGS];
    unsigned int         qsoTotal[MAX_NUM_OF_LOGS];
    unsigned int         qsoPointTotal[MAX_NUM_OF_LOGS];
    unsigned int         multTotal[MAX_NUM_OF_LOGS];
    unsigned long        intervalScore[MAX_NUM_OF_LOGS];
    unsigned long        score[MAX_NUM_OF_LOGS];
    unsigned int         qsoCountInterval[MAX_NUM_OF_LOGS];

    unsigned char        previousBand[MAX_NUM_OF_LOGS];
    unsigned int         bandChanges[MAX_NUM_OF_LOGS];
    unsigned int         totalBandChanges[MAX_NUM_OF_LOGS];
    unsigned char        startBand[MAX_NUM_OF_LOGS];
    
    boolean     done[MAX_NUM_OF_LOGS];
    int         numWorking;
    int         totalQsoCount = 0;

    header_data header[MAX_NUM_OF_LOGS];

    time_t maxTime, minTime;

#if defined(LC_GUI) && defined(LC_NT)
    // These are to remove warnings about these not being used.
    hInstance = hInstance;
    hPrevInstance = hPrevInstance;
    lpCmdLine = lpCmdLine;
    nCmdShow = nCmdShow;
#endif

#if !defined(LC_CMD_LINE)
    MessageBox(NULL, "Amateur Radio Log File Comparison v1.10\n\r\n\r             (c) 2000 by KA5WSS",
               progName, MB_OK);
#else
    printf("Amateur Radio Log File Comparison v1.10\n\r(c) 2000 by KA5WSS\n\r\n\r");

#if defined(LC_GUI) && defined(LC_NT)
    {
        char *CmdLineTmp = new char[strlen(lpCmdLine) + 1];
        char *tmp1, *tmp2, *tmp3;

        strcpy(CmdLineTmp, lpCmdLine);   
        if (((tmp1 = strtok(CmdLineTmp, " ")) == NULL)
             || ((tmp2 = strtok(NULL, " ")) == NULL)
             || ((tmp3 = strtok(NULL, " ")) == NULL))
        {
            if (tmp1 && (stricmp(tmp1, "-h") == 0))
                ShowOptions();
			else
			{
				printf("usage: clogcomp infile1 infile2 outfile [options]\n\r");
				printf("\n\rFor a complete list of options type in \'%s -h\'\n\r", exeName);
			}
            return(1);
        }
        strcpy(InputFileName1, tmp1);
        strcpy(InputFileName2, tmp2);
        strcpy(OutputFileName, tmp3);

        delete CmdLineTmp;
    }
#else    
    if (argc < 4)
    {
        if ((argc == 2) && (stricmp("-h", argv[1]) == 0))
            ShowOptions();
        else
		{
			printf("usage: %s infile1 infile2 outfile [options]\n\r", exeName);
			printf("\n\rFor a complete list of options type in \'%s -h\'\n\r", exeName);
		}
        return(1);
    }

    *lpCmdLine = NULL;
    for (int c = 4; c < argc; c++)     // Turn command line arguments into a string
    {                                   // Similar to Windows.
        strcat(lpCmdLine, argv[c]);
        strcat(lpCmdLine, " ");
    }
    strcpy(InputFileName1, argv[1]);
    strcpy(InputFileName2, argv[2]);
    strcpy(OutputFileName, argv[3]);
#endif    
#endif

#if !defined(LC_CMD_LINE)
    while (bAnotherLog == IDYES)
    {
        if (!GetInputFileName(InputFileName1))
        {
            MessageBox(NULL, "Can't find the requested file", progName, MB_OK | MB_ICONSTOP);
            return FALSE;
        }
#endif
        switch (LogType(InputFileName1))
        {
            case LOG_TYPE_TRLOG:
                    logs[1] = new RTrFile(InputFileName1);
                    break;
            case LOG_TYPE_ARRL:
                    logs[1] = new RArrlFile(InputFileName1);
                    break;
            case LOG_TYPE_UNKNOWN:
            default:
#if !defined(LC_CMD_LINE)            
                    MessageBox(NULL, "File type not supported!", progName, MB_OK | MB_ICONSTOP);
#else
                    printf("File type not supported!\n\r");
#endif                    
                    return FALSE;
        }
        logs[1]->setOptions(lpCmdLine);

#if !defined(LC_CMD_LINE)
        if (!GetInputFileName(InputFileName2))
        {
            MessageBox(NULL, "Can't find the requested file", progName, MB_OK | MB_ICONSTOP);
            return FALSE;
        }
#endif
        switch (LogType(InputFileName2))
        {
            case LOG_TYPE_TRLOG:
                    logs[2] = new RTrFile(InputFileName2);
                    break;
            case LOG_TYPE_ARRL:
                    logs[2] = new RArrlFile(InputFileName2);
                    break;
            case LOG_TYPE_UNKNOWN:
            default:
#if !defined(LC_CMD_LINE)            
                    MessageBox(NULL, "File type not supported!", progName, MB_OK | MB_ICONSTOP);
#else
                    printf("File type not supported!\n\r");
#endif                    
                    return FALSE;
        }
        logs[2]->setOptions(lpCmdLine);

        header[1] = logs[1]->headerInfo();
        header[2] = logs[2]->headerInfo();
        if (header[1].contest != header[2].contest)
        {
#if !defined(LC_CMD_LINE)            
            MessageBox(NULL, "The two input files are not of the same contest!",
                       progName, MB_OK | MB_ICONSTOP);
#else
            printf("The two input files are not of the same contest!\n\r");
#endif                       
            ShutDown();
            return FALSE;
        }

#if !defined(LC_CMD_LINE)        
        FilePrefix(OutputFileName, InputFileName1);
        if (!GetOutputFileName(OutputFileName, &OutputFileType))
        {
            MessageBox(NULL, "Can't open the requested file", progName, MB_OK | MB_ICONSTOP);
            ShutDown();
            return FALSE;
        }
        else
#endif
        {
            outFile = new RFile();
/*            switch (OutputFileType)
            {
                case LOG_TYPE_DBASE:
                        logs[0] = new RDbFile();
                        break;
                default:
#if !defined(LC_CMD_LINE)                
                        MessageBox(NULL, "File type not implemented!", progName, MB_OK | MB_ICONSTOP);
#else
                        printf("File type not implemented!\n\r");
#endif                        
                        return FALSE;
            }*/
            /* Should have exited from switch statement if object not initialized! */
//            if (!logs[0]->open(OutputFileName, ios::out | ios::noreplace | ios::binary))
            if (!outFile->open(OutputFileName, ios::out | ios::noreplace | ios::binary))
            {
#if !defined(LC_CMD_LINE)                
                MessageBox(NULL, "Output file could not be opened", progName, MB_OK | MB_ICONSTOP);
#else
                printf("Output file could not be opened\n\r");
#endif                
                ShutDown();
                return FALSE;
            }
                
        }

/*        if (!(logs[0]->writeHeader(logs[1]->headerInfo())))
        {
            MessageBox(NULL, "Contest not supported by output file", progName, MB_OK | MB_ICONSTOP);
            return FALSE;
        }*/

        ProcessOptions((char *)lpCmdLine, &options);

        minTime = 0;
        maxTime = 0;
        for (int a = 1; a <= 2; a++)
        {
            logs[a]->nextQso(&qso[a]);
            if ((qso[a].getTime() < minTime)
				|| (minTime == 0))
                minTime = qso[a].getTime();
            if (qso[a].getTime() > maxTime)
                maxTime = qso[a].getTime();
        }
        
        if ((maxTime - minTime) > 172800)
        {
#if !defined(LC_CMD_LINE)            
            MessageBox(NULL, "These log files are too far apart in time!",
                       progName, MB_OK | MB_ICONSTOP);
#else
            printf("These log files are too far apart in time!\n\r");
#endif                       
            ShutDown();
            return FALSE;
        }

        sprintf(displayBuffer, "%s log comparison between %s and %s\r\n\r\n",
                contestNames[header[1].contest], header[1].callsign, header[2].callsign);
        outFile->write(displayBuffer, strlen(displayBuffer));

        switch (options.reportType) {
            case REPORT_DEFAULT:
                sprintf(displayBuffer, "%s %9s %23s %10s %18s\r\nSlice\r\n", "Time", "Score", "Score Delta",
                        "QSOs", "Mults");
                outFile->write(displayBuffer, strlen(displayBuffer));
                
                sprintf(displayBuffer, "%13s %8s %8s %8s %8s %8s %8s %8s\r\n", header[1].callsign, header[2].callsign,
                        header[1].callsign, header[2].callsign, header[1].callsign, header[2].callsign,
                        header[1].callsign, header[2].callsign);
                outFile->write(displayBuffer, strlen(displayBuffer));
                break;
            case REPORT_QSOS:
                sprintf(displayBuffer, "%s %6s %17s %9s %18s\r\nSlice\r\n", "Time", "QSOs", "QSO Diff",
                        "Rate", "Rate Diff");
                outFile->write(displayBuffer, strlen(displayBuffer));
                
                sprintf(displayBuffer, "%11s %6s %6s %6s %6s %6s %6s %6s\r\n", header[1].callsign,
                        header[2].callsign, header[1].callsign, header[2].callsign, header[1].callsign,
                        header[2].callsign, header[1].callsign, header[2].callsign);
                outFile->write(displayBuffer, strlen(displayBuffer));
                break;
            case REPORT_BANDS:
                sprintf(displayBuffer, "%s %11s %15s  %11s\r\nSlice\r\n", "Time", "Start Band",
                        "Band Changes", "Total Band Changes");
                outFile->write(displayBuffer, strlen(displayBuffer));
                
                sprintf(displayBuffer, "%11s %6s %6s %6s %6s %6s\r\n", header[1].callsign,
                        header[2].callsign, header[1].callsign, header[2].callsign, header[1].callsign,
                        header[2].callsign);
                outFile->write(displayBuffer, strlen(displayBuffer));
                break;
        };
        
        currTime = ((((minTime / 60) / options.timeSlice) * options.timeSlice) * 60);
        numWorking = 2;
        
        // Initialize variables.
        totalQsoCount = 0;
        for (int x = 1; x < MAX_NUM_OF_LOGS; x++)
        {
            score[x] = qsoTotal[x] = qsoPointTotal[x] = multTotal[x] = 0;
            bandChanges[x] = totalBandChanges[x] = 0;
            previousBand[x] = qso[x].getBand();
            done[x] = FALSE;
        }
        
        while (numWorking > 0)
        {
            struct tm *dosTime;
            
            nextTime = currTime + (60 * options.timeSlice);

            for (int c = 1; c <= 2; c++)
            {
                unsigned long tempScore;
                
                qsoCountInterval[c] = 0;
                
                bandChanges[c] = 0;
                startBand[c] = qso[c].getBand();
                
                while (!done[c] && (qso[c].getTime() < nextTime))
                {
                    qsoTotal[c]++;                      // Count of all QSO's worked
                    qsoCountInterval[c]++;              // QSO's worked in this time slice (rate)
                    qsoPointTotal[c] += qso[c].getQsoPoints();
                    multTotal[c] += qso[c].getMultCount();

                    if (qso[c].getBand() != previousBand[c])
                    {
                        bandChanges[c]++;
                        totalBandChanges[c]++;
                    }
                    previousBand[c] = qso[c].getBand();
                    
                    logs[c]->nextQso(&qso[c]);
                    if (logs[c]->eof())
                    {
                        numWorking--;
                        done[c] = TRUE;
                    }
                }
                tempScore = (unsigned long)qsoPointTotal[c] * (unsigned long)multTotal[c];
                intervalScore[c] = tempScore - score[c];        // How many points scored this time slice?
                score[c] = tempScore;                           // Current running score.

            }
            /* write out interval values for all log files. */
            dosTime = gmtime(&currTime);
            strftime(displayBuffer, 80, "%H%M ", dosTime);
            outFile->write(displayBuffer, strlen(displayBuffer));

            switch (options.reportType) {
                case REPORT_DEFAULT:
                    sprintf(displayBuffer, "%8ld %8ld ", score[1], score[2]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
            
                    sprintf(displayBuffer, "%8ld %8ld ", intervalScore[1], intervalScore[2]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
            
                    sprintf(displayBuffer, "%8d %8d ", qsoTotal[1], qsoTotal[2]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
                    
                    sprintf(displayBuffer, "%8d %8d\r\n", multTotal[1], multTotal[2]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
                    break;
                case REPORT_QSOS:
                    sprintf(displayBuffer, "%6d %6d ", qsoTotal[1], qsoTotal[2]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
                    
                    sprintf(displayBuffer, "%6d %6d ", qsoTotal[1] - qsoTotal[2],
                                                       qsoTotal[2] - qsoTotal[1]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
                    
                    sprintf(displayBuffer, "%6d %6d ", qsoCountInterval[1], qsoCountInterval[2]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
                    
                    sprintf(displayBuffer, "%6d %6d\r\n", qsoCountInterval[1] - qsoCountInterval[2],
                                qsoCountInterval[2] - qsoCountInterval[1]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
                    break;
                case REPORT_BANDS:
                    sprintf(displayBuffer, "%6s %6s ", bandName[startBand[1]], bandName[startBand[2]]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
                    
                    sprintf(displayBuffer, "%6d %6d ", bandChanges[1], bandChanges[2]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
                    
                    sprintf(displayBuffer, "%6d %6d\r\n", totalBandChanges[1], totalBandChanges[2]);
                    outFile->write(displayBuffer, strlen(displayBuffer));
                    
                    break;
            };
            
            currTime = nextTime;
        }
        ShutDown();
        outFile->close();

        for (int b = 1; b <= 2; b++)
            totalQsoCount += qsoTotal[b];
#if !defined(LC_CMD_LINE)            
        sprintf(displayBuffer, "Comparison Complete!\n\rProcessed %d QSO's", totalQsoCount);
        MessageBox(NULL, displayBuffer, progName, MB_OK);

        bAnotherLog = MessageBox(NULL, "Would you like to compare some more log files?",
                                 progName, MB_YESNO | MB_ICONQUESTION);
    }
#else
        printf("Conversion Complete!\n\rProcessed %d QSO's\n\r", totalQsoCount);
#endif                                 
               
    return 0;
}

#if !defined(LC_CMD_LINE)
// Warning!  The buffer passed via sFileName had better be at least as large as STRING_BUFFER_SIZE!

static int GetInputFileName(char *sFileName)
{
    OPENFILENAME OpenFileName;

    memset(&OpenFileName, '\0', sizeof(OPENFILENAME));
    memset(sFileName, '\0', STRING_BUFFER_SIZE);
    OpenFileName.lStructSize = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner = NULL;
    OpenFileName.lpstrFile = sFileName;
    OpenFileName.nMaxFile = STRING_BUFFER_SIZE;
    OpenFileName.lpstrTitle = NULL;
    OpenFileName.lpstrFilter = "ARRL Log files (*.log)\0*.LOG\0TR Log files (*.dat)\0*.DAT\0";
    OpenFileName.lpstrDefExt = "";
    OpenFileName.nFilterIndex = 1L;
    OpenFileName.Flags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST
                         | OFN_HIDEREADONLY | OFN_NOTESTFILECREATE;

    if (!GetOpenFileName(&OpenFileName))
        return FALSE;
    else
        return TRUE;
}

// Warning!  The buffer passed via sFileName had better be at least as large as STRING_BUFFER_SIZE!

static int GetOutputFileName(char *sFileName, int *iFileType)
{
    OPENFILENAME OpenFileName;

    memset(&OpenFileName, '\0', sizeof(OPENFILENAME));
    OpenFileName.lStructSize = sizeof(OPENFILENAME);
    OpenFileName.hwndOwner = NULL;
    OpenFileName.lpstrFile = sFileName;
    OpenFileName.nMaxFile = STRING_BUFFER_SIZE;
    OpenFileName.lpstrTitle = NULL;
    OpenFileName.lpstrFilter = "Text Files (*.txt)\0*.TXT\0";
    OpenFileName.lpstrDefExt = "";
    OpenFileName.nFilterIndex = 1L;
    OpenFileName.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_NOTESTFILECREATE;

    if (!GetSaveFileName(&OpenFileName))
        return FALSE;
    else
    {
        boolean AppendExt = FALSE;

        // Under Win3.1 the extension is not automatically appended to the filename.
        //  Do it manually, if needed.        
        if ((OpenFileName.nFileExtension == 0) || (OpenFileName.nFileExtension == strlen(sFileName)))
            AppendExt = TRUE;
            
//        switch (OpenFileName.nFilterIndex)
//        {
//            case 1:
                *iFileType = LOG_TYPE_UNKNOWN;
                if (AppendExt)
                    strcat(sFileName, ".txt");
//                break;
//            default:
//                *iFileType = LOG_TYPE_UNKNOWN;
//        }
                
        return TRUE;
    }
}

// This routine returns the prefix name of a file supplied in pathName.  For
//  example pathName="d:\tmp\example.txt" would result in name="example".
static void FilePrefix(char *name, char *pathName)
{
    char *dot = strrchr(pathName, '.');
    char *slash = strrchr(pathName, '\\');

    if (!dot)
        dot = pathName + strlen(pathName);
    if (!slash)
        slash = pathName;
    else
        slash++;

    strncpy(name, slash, dot - slash);
    name[dot - slash] = NULL;
    return;
}
#endif /* !LC_CMD_LINE */

static int LogType(char *sFileName)
{
    RFile Log(sFileName);

    return Log.fileType();
}

static void ShutDown()
{
    for (int i = 0; i < MAX_NUM_OF_LOGS; i++)
    {
        if (logs[i] != NULL)
        {
            logs[i]->close();
            delete logs[i];
        }
    }
    return;
}

static void ProcessOptions(char *cmdLine, struct optType *options)
{
    char *tmpPtr;
    
    if ((tmpPtr = strstr(cmdLine, "-t")) != NULL)       // Set time slice
    {
        tmpPtr += 2;
        options->timeSlice = atoi(tmpPtr);
    }
    else
        options->timeSlice = 60;                        // Default is 60 minutes

    if (strstr(cmdLine, "-oqsos") != NULL)
        options->reportType = REPORT_QSOS;
    else if (strstr(cmdLine, "-obands") != NULL)
        options->reportType = REPORT_BANDS;
    else
        options->reportType = REPORT_DEFAULT;
}

#if defined(LC_CMD_LINE)
static void ShowOptions()
{
    printf("usage: %s infile1 infile2 outfile [options]\n\r\n\r", exeName);
    printf("  -oqsos       Generate QSO rate report.\n\r"); 
    printf("  -obands      Generate band usage, band change report.\n\r");
	printf("\n\rThe default report presents QSO, mult and score information.\n\r");
    return;
}
#endif

